	titLE	'SETDRIVE '
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;									;
; Program Name:		SETDRIVE					;
; Author:		L. Lammi					;
;									;
; Date Originated:	01-15-84					;
; 	Rev 1.0		01-16-84     LEL				;
;	Rev 1.1		01-27-84     LEL				;
;	Rev 1.2  	03-27-84     LEL				;
;	Rev 1.2.1       06-20-84     LEL                                ;
;	Rev 1.3		11-20-86     CMS				;
; This program allows the user to specify the type of media in each	;
; drive.  Media types currently supported are Amigo single-sided,	;
; Osborne I, Morrow, Visual 1050, Kaypro, VT180, and DEC Rainbow.	;
; Rev 1.3 Allows changing from single-sided to double-sided   		;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	ASEG
	ORG	100h

	MACLIB	Z80

;
; Equates
;

bdos		equ	5	; BDOS
drbios		equ	50	; Direct bios function call
seldsk		equ	9	; Bios select disk function
num$drives	equ	2	; Number of drives
tab$length	equ	16	; Table entry length
num$entries	equ	8	; # table entries		(CMS 1186)
cmd$tail	equ	080h	; Address of command tail             
cr		equ	13	; carriage return
lf		equ	10	; line feed
tab		equ	09	; horizontal tab
bs		equ	08	; backspace
esc		equ	01bh	; escape
	page

SETDRIVE:

	lxi	d,intro		; Print introductory message
	call	pstr		; 

;
; Two forms of the command are allowed: inline and prompted.  See
; which one you have.
;


	lda	cmd$tail      	; Get command tail length
	ora	a  		; Is it 0?
	jnz	setdr$50	; Jump if not

;
; Prompted version.  Begin by displaying current settings for each
; of the drives.
;

	lxi	d,setting$msg	; Point to message
	call	pstr		; print it out

	xra	a  		; Start with A drive
SETDR$7:
	cpi	num$drives  	; Compare with # drives
	jp	setdr$12	; Jump out when done
	call	display		; If valid, display current value
	inr	a		; Increment to next drive
	jr	setdr$7		; Loop till done

;
; See if the user wants to make a change to the current settings
;

SETDR$12:
	lxi	d,change$msg	; Point to question
	call	pstr		; ask it

	call	get$answer	; Get user response
	cpi	'N'		; if N, exit
	jz	setdr$98	;
	cpi	cr		; If cr, exit
	jz	setdr$98	;
	cpi	' '		; If space, exit
	jz	setdr$98	;

	cpi	'Y'		; OK if Y
	jrnz	setdr$12	; Else, keep asking

;
; Ask the user which drive is to be changed.
;

SETDR$16:  
	lxi	d,drive$msg	; Point to question
	call	pstr		; ask it

	mvi	a,'B'		; Default to B drive
	sta	dr$code		; Store as drive code

	call	get$answer	; Get user's response
	cpi	cr		; If cr, use default
	jrz	setdr$22	;
	cpi	' '		; If space, exit
	jz	setdr$98	;

	sta	dr$code		; Store drive code
	cpi	num$drives+41h	; Compare with # drives
	jp	setdr$80	; Jump if invalid
	cpi	'A'		; Make sure it's alpha
	jm	setdr$80	; Jump if not

;
; Ask the user to specify the new media type.
;

SETDR$22:
	lxi	d,media$msg	; Point to question
	call	pstr		; ask it

	mvi	a,'V'		; Default to 1050 media
	sta	media  		; Store as media     

	call	get$answer	; Get user's response
	
	push	psw		; save answer
	lxi	d,crlf		; Do a crlf
	push	d		; save
	call	pstr		;
	pop	d		; Do another
	call	pstr		;
	pop	psw		; restore answer

	cpi	' '		; If space, exit
	jz	setdr$98	;
	cpi	cr		; If cr, use default
	jrz	setdr$30	;

	sta	media  		; Store media
SETDR$30:
	call	media$lookup	; Make sure it's valid

	jc	setdr$82	; If not, print error msg

;
; Move appropriate disk info into parameter table.
;

SETDR$32:
	inx	h		; Point to Translate 
	push	h		; Save it

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;									;
;		COMMENTED OUT  03/84       (LEL)			;
;									;
;									;
; Now, have user insert  new disk (unless 1050)				;
;									;
;									;
;	lxi	b,0-12        	; Is pointing to translate 		;
;	dad	b		; So, back it up			;
;									;
;	lxi	d,type$tab	; Get V1050 address			;
;	push	h		; Save current entry			;
;	ora	a		; clear carry				;
; 	DSBC	DE		; Was it changed to 1050?		;
;	pop	h		; (Restore entry)			;
;	jz	setdr$38	; If yes, don't ask for disk		;
;									;
;	lxi	b,11		; Move disk type into message		;
;	lxi	d,insert$type	;					;
; 	ldir			; move in				;
;									;
;	lda	dr$code		; Get drive code			;
;	sta	insert$dr	; Put into message			;
;									;
;	lxi	d,insert$msg	; Point to message			;
;SETDR$35:								;
;	call	pstr		; print it				;
;									;
;	call	get$answer	; get user response			;
;	cpi	' '		; If space, exit			;
;	jz	setdr$98	;					;
;	cpi	cr		; Is it CR?				;
;	jrz	setdr$38	; Jump if yes     			;
;	lxi	d,re$insert	; point to message			;
;	jr	setdr$35	; loop back				;
;									;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 
;  
; Select drive to get DPH info
;
SETDR$38:
	lda	dr$code		; Get drive to be affected
	sui	41h		; make numeric
	sta	creg		; store for call
	call	select		; Select drive

;
; Upon return, DPH address is in HL
; First, Get new Translate table address.
;
	mvi	a,30		; Make private BIOS call
	sta	biosfc		; Store function code
	pop	d		; Restore Translate key
	ldax	d 		; Move to A
	sta	C$reg		; Store for call
	inx	d		; Increment to TYPE field
	push	d		; save for later
	push	h		; save DPH address
	mvi	a,02		; Bios function code
	sta	a$reg		; Store for call
	lxi	d,biospb	; point to param block
	mvi	c,drbios	; Make direct bios call
	call	bdos		;
;
; Upon return, new Translate table address is in HL.
;
	pop	d		; Get DPH address
	xchg			; switch
	mov	m,e		; Store new xlate address
	inx	h		;
	mov	m,d		;

;
; Then, store new TYPE field.
;

	dcx	h		; Decrement to point to TYPE field
	dcx	h		;
	pop	d		; Get address of new TYPE
	ldax	d		; get new TYPE
	mov	m,a		; Store it

;
; Now, move in new DPB.
;
	inx	d		; Increment DE to point to DPB addr
	push	h		; save DPH address
	xchg			; Exchange DE and HL
	mov	e,m		; Get address into DE
	inx	h		;
	mov	d,m		; New DPB values at DE
	pop	h		; HL = DPB address
	push	d 		; Save address of new DPB values

	lxi	d,13		; Load offset to DPB address
	dad	d		; add to start of DPH
	mov	e,m		; Now move DPB into DE
	inx	h		;
	mov	d,m		; DE now contains DPB addr
	pop	h		; HL = DPB address
	lxi	b,17		; Load length of DPB
	ldir			; Move in

;
; Reset all drives
;
	mvi	c,13		; BDOS call 13= Reset disk system
	call	bdos		; perform call

	jmp	setdr$98	; DONE				(LEL0384)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;									;
;		COMMENTED OUT 03/84  (LEL)				;
;									;
; Now, try to verify that the disk parameters are correct		;
; by selecting the drive (will try to read directory).			;
; 									;
;									;
;SETDR$44:								;
;	lda	media		; Check media type			;
;	cpi	'V'		; Visual 1050?				;
;	jz	setdr$98	; If yes, skip it			;
;									;
;									;
; Set error mode so we can see if there's an error			;
;									;
;									;
;	mvi	c,45		; SET error mode BDOS call		;
;	mvi	e,0ffh		; Send back error code			;
;	call	bdos		; Perform call				;
;									;
;									;
; Now, select the drive							;
;									;
;									;
;	lda	dr$code		; Get drive code			;
;	sui	41h		; Make numeric				;
;	mov	e,a 		; Store for call			;
;	mvi	c,14		; BDOS SELECT disk			;
;	call	bdos   		; select it				;
;									;
;									;
; Check error code							;
;									;
;									;
;	ora	a		; Is it 0?				;
;	jrz	SETDR$90	; jump if yes				;
;									;
;									;
; Oops.  Not right.							;
;									;
;	lxi	d,disk$error	; Point to error msg			;
;	call	pstr		; display it				;
;									;
;	jmp	setdr$16	; Ask again				;
;									;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;
; Inline version.  Get values from command line to fill in DR$code
; and MEDIA.
;
SETDR$50:
	mvi	a,'V'		; Default to 1050
	sta	media		; if media not specified
	lxi	h,cmd$tail  	; Point to beginning of command tail
	mov	b,m		; Store count in b-reg
setdr$51:
	inx	h		; Point to next char
	mov	a,m		; Get char
	cpi	' '		; Is it space?
	jrz	setdr$52
	call	xlate		; Translate to upper case
	cpi	num$drives+41h	; Valid drive?
	jp	setdr$80	; If not, error
	cpi	'A'		; Is it alphabetic?
	jm	setdr$80	; If not, error
	sta	dr$code		; Else, store drive code
	dcr	b		; decrement count
	jz	setdr$30	; Jump if done
	jr	setdr$53	; Else, get media
setdr$52:
	dcr	b		; Decrement count
	jz	setdr$16	; If all spaces, then ask
	jr	setdr$51
;
; Assume user has used the form SETDRIVE D:M , where D is the
; drive and M is the media
;
setdr$53:
	inx	h		; Point to next char
	mov	a,m		; Get char
	cpi	' '		; If space
	jrz	SEtdr$55	; then jump
	cpi	':'		; If ':'
	jrz	setdr$55	; Then jump
	call	xlate		; Translate to upper case
	sta	media		; Else, store it
	jmp	setdr$30	; and go!

setdr$55:
	djnz	setdr$53	; loop
	jmp	setdr$30

;
; Error handling
;

SETDR$80:
	lxi	d,bad$drive	; Point to error message
	call	pstr		; Print it
	jmp	setdr$16	; Then ask again

SETDR$82:
	lxi	d,bad$media	; Point to error messgae
	call	pstr		; print it
	jmp	setdr$22	; then ask again

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   
;									;
;		COMMENTED OUT 03/84    (LEL)				;
;									;
;SETDR$90:								;
;									;
; Put error mode back to displaying errors        			;
;									;
;									;
;	mvi	c,45		; SET error mode BDOS call		;
;	mvi	e,000h		; Send back error code			;
;	call	bdos		; Perform call				;
;									;
; Do a couple of cr-lfs.						;
;									;
;	lxi	d,crlf		; Point to CRLF message			;
;	push	d		; Save address				;
;	call	pstr		; Do one				;
;	pop	d		;					;
;	call	pstr		; Then another				;
;									;
;									;
; Link to do a DIR before leaving					;
;									;
;									;
;	lxi	d,80h		; Put at Default DMA addr		;
;	lxi	h,DIR$cmd	; Point to command			;
;	lda	dr$code		; Get drive code			;
;	sta	dir$dr		; store in command stream		;
;	lxi	b,7		; Command length			;
;	ldir			; move in				;
;	mvi	c,47		; BDOS chain to program			;
;	mvi	e,0		;					;
;	call	bdos		;					;
;									;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SETDR$98:
	mvi	c,13		; Reset all drives		(LEL0184)
	call	bdos		;				(LEL0184)
	JMP	0

	page
;
; SUBROUTINES
;
DISPLAY:
;
; Display current drive settings
;

	push	psw		; Save current drive #
	sta	creg		; Set up C reg in param block
	adi	41h		; Turn into ASCII
	sta	drive		; set up message

;
; Determine current drive setting by looking at the TYPE field
; of the DPH.  To get DPH address, perform a direct bios call
; to select the disk.
;

	call	select		; select drive

;
; Upon return, DPH address is in HL.  TYPE field is at the
; current address-1.  Bits 7,6,5 are used to determine the disk
; format as follows:
;
;     BIT 7   BIT 6  BIT 5		DISK TYPE
;
;	0	0	0		Visual 1050
;	0	0	1		Kaypro
;	0	1	0		Amigo
;	0	1	1		Rainbow
;	1	0	0		VT180
;       1       0       1               Morrow
;       1       1       0               Osborne I               
;	1	1	1		Visual 1050 double-sided
;

	dcx	h		; Point at TYPE field
	mov	a,m		; Get TYPE
	rlc			; Rotate into bits 0-2
	rlc			;
	rlc			;
	ani	0000$0111b	; Mask out other bits
	jrz	display$30	; Jump if 0

;
; Get ready to set up message.  Use TYPE field as index
; into display table
;

	mov	b,a		; Move TYPE into B-reg for count
	xra	a		; Clear a
DISPLAY$25:
	adi	tab$length	; add table entry length        
	djnz	display$25	; Loop B times

;
; Move type text into display message.
;

DISPLAY$30:
	mov	c,a		; Move TYPE*11 into C
	mvi	b,0		; Clear B
	lxi	h,type$tab	; Point to start of table
	dad	b		; add offset
	lxi	b,11		; message length
	lxi	d,type		; Point to destination
	ldir			; Move message

;
; Display message.
;

	lxi	d,disp$msg	; Point to message
	call	pstr		; Print it
	pop	psw		; Restore drive code
	ret

;
; SELECT subroutine.
; This routine performs the SELDSK direct bios call.
;

SELECT:
	lxi	h,0ffffh	; Ereg must be non-zero
	shld	ereg		; Store in param block
	mvi	a,seldsk	; Bios function= Select Disk
	sta	biosfc		; Store bios function code
DIRECT:
	mvi	c,drbios	; Set up direct bios call
	lxi	d,biospb	; Point to parameter block
	call	bdos		; call BDOS
	ret			; then return

;
; Media type lookup routine.
;
MEDIA$LOOKUP:
	lda	media		; Get users choice
	mvi	b,Num$entries	; # media types supported
	lxi	d,tab$length	; table entry length
	lxi	h,type$tab+11	; Point at first valid entry
MEDIA$LOOP:
	cmp	m		; Compare with acc.
	rz	        	; return if good
	dad	d		; point to next entry
	djnz	media$loop	; loop thru table
	stc			; Set carry if no match
	RET			;

;
; Print string subroutine
;
PSTR:
	push	h		; save H
	mvi	c,9		; Bdos funcion 9=print string
	call	bdos		; perform call
	pop	h		; restore h
	ret			; and return

;
; Get user response subroutine.
;

GET$ANSWER:
	mvi	c,1		; Bdos call 1
	call	bdos		; Get a charcater

	push	psw
	lxi	d,crlf		; Send a CR-LF
	call	pstr
	pop	psw		; Restore char

XLATE:
	cpi	'a'		; If lower case, make upper case
	rc			;
	cpi	'z'+1		;
	rnc			;
	sui	20h		;
	ret			;
	page
;
; Messages
;

INTRO:
	db	cr,lf 
	db	esc,'[1m'
	db	'SETDRIVE V1.3',esc,'[m',cr,lf,'$'

SETTING$MSG:
	db	cr,lf,'CURRENT SETTINGS:  ',cr,'$'
DISP$MSG:
	db	esc,';H',tab,tab,tab,esc,';@'
DRIVE:
	db	0
	db	': '
TYPE:
	db	'           '
	db	cr,lf,'$'
;
; The following table contains the required info for each media type.
; Each table entry contains:
;
;	Media name
;	User key field
;	Key for Translate table (00 = none, 01=1050, 02=Rainbow, 03=Morrow)
;	TYPE field for DPH (See description following)
;	Address of DPB info
;

TYPE$TAB:
	DB	'VISUAL 1050'
	DB		'V'
	DB		01
	DB		0000$0010b
	DW		V1050$DPB

	DB	'KAYPRO     '
	DB		'K'
	DB		00
	DB		20H
	DW		KAYPRO$DPB

	DB	'AMIGO SS   '
	DB		'A'
	DB		01
	DB		40H
	DW		AMIGO$DPB

	DB	'RAINBOW    '
	DB		'R'
	DB		02
	DB		62H
	DW		RAIN$DPB

	DB	'DEC VT180  '
	DB		'D'
	DB		02
	DB		80H
	DW		VT180$DPB

	DB	'MORROW     '		;			(LEL0684)
	DB		'M'		;			(LEL0684)
	DB		03		;			(LEL0684)
	DB		1010$0000b	;			(LEL0684)
	DW		MorrowDPB	;			(LEL0684)

	DB	'OSBORNE I  '		;			(LEL0684)
	DB		'O'		;			(LEL0684)
	DB		02         	;			(LEL0684)
	DB		1100$0100b	;			(LEL0684)
	DW		OsborneDpb	;			(LEL0684)

	DB	'VISUAL DS  '		;			(CMS 1186)
	DB		'T'		;			(CMS 1186)
	DB		01		;			(CMS 1186)
	DB		1110$0011b	;			(CMS 1186)
	DW		VisdsDPB	;			(CMS 1186)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                  ;
; TYPE field of XDPH is specified as follows:                      ;
;                                                                  ;
; Bits 7,6,5 are used to determine the disk			   ;
; format as follows:						   ;
;							           ;
;     BIT 7   BIT 6  BIT 5		DISK TYPE                  ;
;                                                                  ;
;	0	0	0		Visual 1050                ;
;	0	0	1		Kaypro                     ;
;	0	1	0		Amigo                      ;
;	0	1	1		Rainbow                    ;
;	1	0	0		VT180                      ;
;       1       0       1               Morrow                     ;
;       1       1       0               Osborne I                  ;
;	1	1	1		Visual double-sided	   ;
;                                                                  ;
; Other bits are:                                                  ;
;                                                                  ;
;    BIT 4   -  Unused                                             ;
;    BIT 3   -  Unused                                             ;
;    BIT 2   -  Single (1) or Double (0) Density                   ;
;    BIT 1   -  48 (0) or 96 (1) TPI                               ;
;    BIT 0   -  Double Sided					   ;
;                                                                  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;
; DPB for Ontel Amigo (single sided)
;
AMIGO$DPB:
        dw     40              ;#128 byte records/track
        db     3,07h           ;block shift mask     
        db     0               ;extent  mask  
        dw     189             ;maximun  block number
        dw     63              ;max number of dir entry - 1
        db     0C0H,00h        ;alloc vector for directory
        dw     16              ;checksum size
        dw     2               ;offset for sys tracks
        db     2,3             ;physical sector shift (512 sector)
;
; DPB for DEC Rainbow and V1050
;
V1050DPB:
RAINDPB:
        dw     028h            ;#128 byte records/track
        db     4,0fh           ;block shift mask (2K)
        db     1               ;extent  mask  
        dw     194             ;maximun  block number
        dw     127             ;max number of dir entry - 1
        db     0C0H,00h        ;alloc vector for directory
        dw     0020h           ;checksum size
        dw     2               ;offset for sys tracks
        db     2,3             ;physical sector shift (512 sector)
;
; DPB for Visual double-sided
;
VisdsDPB:
        dw     028h            ;#128 byte records/track
        db     4,0fh           ;block shift mask (2K)
        db     0               ;extent  mask  
        dw     394             ;maximun  block number
        dw     127             ;max number of dir entry - 1
        db     0C0H,00h        ;alloc vector for directory
        dw     0020h           ;checksum size
        dw     2               ;offset for sys tracks
        db     2,3             ;physical sector shift (512 sector)

;
; DPB for DEC VT180
;
VT180$dpb:
        dw     36              ;#128 byte records/track
        db     3,7             ;block shift mask (2K)
        db     0               ;extent  mask  
        dw     170             ;maximum  block number
        dw     63              ;max number of dir entry - 1
        db     0C0H,00h        ;alloc vector for directory
        dw     16              ;checksum size
        dw     2               ;offset for sys tracks
        db     2,3             ;physical sector shift (512 sector)

;
;DPB for KAYPRO II single side 48 tpi
;

kayprodpb:
	dw	28h            ;number of 128 records/track
	db	3,7	       ;block shift mask
	db	0	       ;extent mask
	dw	194	       ;max block number
	dw	63	       ;max dir entries - 1
	db      0f0h,0         ;alloc vector for directory
	dw	0010h          ;checksum size
	dw	1	       ;offset sys tracks
	db	2,3	       ;physical sector shift (512)
;
;DPB for MORROW SSDD                 
;

Morrowdpb:	  	       ;				(LEL0684)
	dw	28h            ;number of 128 records/track	(LEL0684)
	db	4,0fh	       ;block shift mask		(LEL0684)
	db	1	       ;extent mask			(LEL0684)
	dw	094	       ;max block number		(LEL0684)
	dw	127	       ;max dir entries - 1		(LEL0684)
	db      0c0h,0         ;alloc vector for directory	(LEL0684)
	dw	0020h          ;checksum size			(LEL0684)
	dw	2	       ;offset sys tracks		(LEL0684)
	db	3,7	       ;physical sector shift (1024)	(LEL0684)

;
;DPB for Osborne I                  
;

osbornedpb:                    ;				(LEL0684)
	dw	14h            ;number of 128 records/track	(LEL0684)
	db	4,0fh	       ;block shift mask		(LEL0684)
	db	1	       ;extent mask			(LEL0684)
	dw	02dh	       ;max block number		(LEL0684)
	dw	63	       ;max dir entries - 1		(LEL0684)
	db      080h,0         ;alloc vector for directory	(LEL0684)
	dw	0010h          ;checksum size			(LEL0684)
	dw	3	       ;offset sys tracks		(LEL0684)
	db	1,1	       ;physical sector shift (256)	(LEL0684)
	

CHANGE$MSG:
	db	cr,lf,'Change current setting? [N]',08h,08h,'$'

BAD$DRIVE:
	db	esc,'[7m','Invalid drive',esc,'[m',cr,lf,'$'

DRIVE$MSG:
	db	cr,lf,'Drive? [B]',08h,08h,'$'

CRLF	db	cr,lf,'$'

MEDIA$MSG:
	db	cr,lf
	db	'Media Type ? [V]',cr,lf
	db	'A=Amigo SS',tab,'V=Visual 1050',tab
	db	'R=Rainbow',tab,'O=Osborne I',cr,lf             ; (LEL0684)
	db	'K=Kaypro',tab,'D=DEC VT180',tab,'M=Morrow',tab	; (LEL0684)
	db	'T=Visual DS',cr				; (CMS 1186)
	db	esc,'[A',01bh,'[A'          
	db	esc,';H',tab,tab,'    '      
	db	esc,';@'
	db	'$'

RE$INSERT:
	db	esc,'[A',esc,'[A'
	db	esc,'[A',esc,'[A'
INSERT$MSG:
	db	cr,lf,'Insert ',esc,'[1m'
INSERT$TYPE:
	ds	11
	db	esc,'[m disk into the ',esc,'[1m'
INSERT$DR:
	db	' '
	db	esc,'[m'
	db	': drive',cr,lf,'Press Retn when ready',cr,lf
	db	esc,'[K'
	db	'Press SPACE to abort$'
BAD$MEDIA:
	db	esc,'[7m','Error in media type',esc,'[m',cr,lf,'$'

DISK$ERROR
	db	esc,'[7m'
	db	'DISK FORMAT ERROR'
	db	esc,'[m',13,10,'$'

DIR$CMD:
	db	'DIR '
DIR$DR:
	DB	'A:'
	db	0
;
; VARIABLES
;

BIOSPB:				; Bios parameter block
BIOSFC:	db	0		; Bios function number
AREG:	db	0		; A register contents
CREG:	db	0		; C register contents
BREG:	db	0		; B register contents
EREG:	db	0		; E register contents
DREG:	db	0		; D register contents
HLREG:	db	0		; HL register contents

DR$CODE:	
	ds	1		; Drive to be changed
MEDIA:
	ds	1


	END
